import { render, screen, fireEvent } from '@testing-library/react';
import TreeGrid from "./TreeGrid"
import React from "react"
import { TreeGridComponent } from '@syncfusion/ej2-react-treegrid';

const mGetRandomValues = jest.fn().mockReturnValueOnce(new Uint32Array(10));
Object.defineProperty(window, 'crypto', {
    value: { getRandomValues: mGetRandomValues },
}); //getRandomValue is mocked as it was called in syncfusion/treegrid and throwing err 

describe('rendering of components', () => {
    test('treegrid component is rendered', () => {
        render(<TreeGrid />)
    });
    test('Syncfusion treegrid is rendered', () => {
        render(<TreeGridComponent />)
    });
})

describe('Treegrid wrapper behaviour', () => {
    test('expand button is rendered intitially', () => {
        const { queryByTestId, queryByPlaceholderText } = render(<TreeGrid />)
        expect(queryByTestId("expandbutton")).toBeTruthy();
    });
    test('collapsed button is rendered when grid is expanded', () => {
        const { queryByTestId, queryByPlaceholderText } = render(<TreeGrid />)
        fireEvent.click(screen.queryByTestId("expandbutton"));
        expect(queryByTestId("collapsebutton")).toBeTruthy();
        expect(queryByTestId("expandbutton")).toBeFalsy();
    });
    test('width of wrapper is collapsed initially', () => {
        render(<TreeGrid />)
        expect(screen.getByTestId("treegrid-container")).toHaveClass("TreeGrid-container-short")
    })
    test('width of wrapper is expanded when expand button is clicked', () => {
        render(<TreeGrid />)
        fireEvent.click(screen.queryByTestId("expandbutton"));
        expect(screen.getByTestId("treegrid-container")).toHaveClass("TreeGrid-container-expanded")
    })
})


describe('Treegrid header behaviour', () => {
    test('header text is hidden when collapsed', async () => {
        const { container } = render(<TreeGrid />)
        await new Promise((r) => setTimeout(r, 100));
        expect(container.getElementsByClassName('hide-treegridheader').length).toBeGreaterThan(0);
    });
    test('header text is visible when expanded', () => {
        const { container } = render(<TreeGrid />)
        fireEvent.click(screen.queryByTestId("expandbutton"));
        expect(container.getElementsByClassName('show-treegridheader').length).toBeGreaterThan(0);
    });
})

describe('Treegrid content overflow behaviour', () => {
    test('overflow is hidden when collapsed', async () => {
        const { container } = render(<TreeGrid />)
        await new Promise((r) => setTimeout(r, 100));
        expect(container.getElementsByClassName('hide-contentoverflow').length).toBeGreaterThan(0);
    });
    test('overflow is auto when expanded', () => {
        const { container } = render(<TreeGrid />)
        fireEvent.click(screen.queryByTestId("expandbutton"));
        expect(container.getElementsByClassName('show-contentoverflow').length).toBeGreaterThan(0);
    });
})

describe.skip('Treegrid border behaviour', () => {
    test('vertical border is hidden when collapsed', async() => {
        const { container } = render(<TreeGrid />)

        fireEvent.click(screen.queryByTestId("expandbutton"));
        fireEvent.click(screen.queryByTestId("collapsebutton"));
        await new Promise((r) => setTimeout(r, 1000));
        expect(container.getElementsByClassName('hide-treegridborder').length).toBeGreaterThan(0);
    });
    test('vertical border is visible when expanded', async() => {
        const { container } = render(<TreeGrid />)
        fireEvent.click(screen.queryByTestId("expandbutton"));
        await new Promise((r) => setTimeout(r, 1000));
        expect(container.getElementsByClassName('show-treegridborder').length).toBeGreaterThan(0);
    });
})
